﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;
using System.Runtime.CompilerServices;
using static CH347PAR.CH347DLL;
using System.Security.Cryptography;

namespace CH347PAR
{
    //由于vc版本的头文件中含有部分宏定义，在C#中无法实现（其中一部分转为常量），完整版请参考vc版CH341DLL.H
    [StructLayout(LayoutKind.Explicit, CharSet = CharSet.Ansi, Size = 8)]
    public struct mUSB_SETUP_PKT                // USB控制传输的建立阶段的数据请求包结构
    {				
        [FieldOffset(0)]
        public byte mUspReqType;                // 00H 请求类型
        [FieldOffset(1)]
        public byte mUspRequest;                // 01H 请求代码

        [FieldOffset(2)]
        public byte mUspValueLow;				// 02H 值参数低字节
        [FieldOffset(3)]
        public byte mUspValueHigh;              // 03H 值参数高字节
        [FieldOffset(2)]
        public ushort mUspValue;                // 02H-03H 值参数

        [FieldOffset(4)]
        public byte mUspIndexLow;				// 04H 索引参数低字节
        [FieldOffset(5)]
        public byte mUspIndexHigh;              // 05H 索引参数高字节
        [FieldOffset(4)]
        public ushort mUspIndex;                // 04H-05H 索引参数
        [FieldOffset(6)]
        public ushort mLength;					// 06H-07H 数据阶段的数据长度
    }

    /*
     mWIN32_COMMAND结构体C++定义如下
     
     typedef	struct	_WIN32_COMMAND {	    // 定义WIN32命令接口结构
         union	{
             ULONG		mFunction;			    // 输入时指定功能代码或者管道号
             NTSTATUS	mStatus;			    // 输出时返回操作状态
         };
         ULONG			mLength;			    // 存取长度,返回后续数据的长度
         union	{
             mUSB_SETUP_PKT	mSetupPkt;		    // USB控制传输的建立阶段的数据请求
             UCHAR			mBuffer[ mCH341_PACKET_LENGTH ];	// 数据缓冲区,长度为0至255B
         };
     } mWIN32_COMMAND, *mPWIN32_COMMAND;

    C#中union功能由FieldOffset()代替

    由于上述结构体中最后一个union可能引起错误“对象字段由一个非对象字段不正确地对齐或重叠”
    因此将上述结构体拆分成2个结构体mWIN32_COMMAND_USB_SETUP_PKT与mWIN32_COMMAND_mBuffer，选其一即可。
    CH341DriverCommand函数将重载
    */

    [StructLayout(LayoutKind.Explicit, CharSet = CharSet.Ansi)]
    public struct mWIN32_COMMAND_USB_SETUP_PKT  // 定义WIN32命令接口结构
    {               
        [FieldOffset(0)]
        public uint mFunction;			        // 输入时指定功能代码或者管道号
        [FieldOffset(0)]
        public Int32 mStatus;                   // 输出时返回操作状态
        [FieldOffset(4)]
        public uint mLength;                    // 存取长度,返回后续数据的长度
        [FieldOffset(8)]
        public mUSB_SETUP_PKT mSetupPkt;        // USB控制传输的建立阶段的数据请求
    }

    [StructLayout(LayoutKind.Explicit, CharSet = CharSet.Ansi)]
    public struct mWIN32_COMMAND_mBuffer        // 定义WIN32命令接口结构
    {				
        [FieldOffset(0)]
        public uint mFunction;				    // 输入时指定功能代码或者管道号
        [FieldOffset(0)]
        public Int32 mStatus;					// 输出时返回操作状态
        [FieldOffset(4)]
        public uint mLength;					// 存取长度,返回后续数据的长度

        [MarshalAs(UnmanagedType.ByValArray, SizeConst = CH347DLL.mCH341_PACKET_LENGTH), FieldOffset(8)]
        public byte[] mBuffer;	// 数据缓冲区,长度为0至255B

    }

    [StructLayout(LayoutKind.Sequential, Pack = 1)] // 结构体使用1字节对齐
    public struct SPI_CONFIG                // SPI控制器配置
    {
        public byte iMode;                  // 0-3: SPI Mode0/1/2/3
        public byte iClock;                 // 0=60MHz, 1=30MHz, 2=15MHz, 3=7.5MHz, 4=3.75MHz, 5=1.875MHz, 6=937.5KHz, 7=468.75KHz
        public byte iByteOrder;             // 0=低位在前(LSB), 1=高位在前(MSB)
        public ushort iSpiWriteReadInterval;// SPI接口常规读取写入数据命令, 单位为us
        public byte iSpiOutDefaultData;     // SPI读数据时默认输出数据
        public uint iChipSelect;            // 片选控制, 位7为0则忽略片选控制, 位7为1则参数有效: 位1位0为00/01分别选择CS1/CS2引脚作为低电平有效片选
        public byte CS1Polarity;            // 位0:片选CS1极性控制: 0:低电平有效; 1:高电平有效;
        public byte CS2Polarity;            // 位0:片选CS2极性控制: 0:低电平有效; 1:高电平有效;
        public ushort iIsAutoDeativeCS;     // 操作完成后是否自动撤消片选
        public ushort iActiveDelay;         // 设置片选后执行读写操作的延时时间, 单位us
        public uint iDelayDeactive;         // 撤消片选后执行读写操作的延时时间, 单位us
    }
    // SPI_CONFIG mPSpiCfgs = new SPI_CONFIG();     // 为什么原.H文件中需要这个指针？

    [StructLayout(LayoutKind.Sequential, Pack = 1)] // 结构体使用1字节对齐
    public struct DEV_INFOR
    {
        public byte iIndex;                 // 当前打开序号
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 64)]
        public byte[] DevicePath;           // 设备链接名, 用于CreateFile
        public byte UsbClass;               // 驱动类别 0: CH347_USB_CH341, 2: CH347_USB_HID, 3: CH347_USB_VCP
        public byte FuncType;               // 功能类别 0: CH347_FUNC_UART, 1: CH347_FUNC_SPI_I2C, 2: CH347_FUNC_JTAG_I2C
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 64)]
        public short[] DeviceID;            // USB\VID_xxxx&PID_xxxx
        public byte ChipMode;               // 芯片工作模式： 0:Mode0(UART0/1); 1:Mode1(Uart1+SPI+I2C);
                                            // 2:Mode2(HID Uart1+SPI+I2C) 3:Mode3(Uart1+Jtag+IIC) 4:CH347F(Uart*2+Jtag/SPI/IIC)
        public IntPtr DevHandle;            // 设备句柄
        public ushort BulkOutEndpMaxSize;   // 批量上传端点大小
        public ushort BulkInEndpMaxSize;    // 批量下传端点大小
        public byte UsbSpeedType;           // USB速度类型，0:FS, 1:HS, 2:SS
        public byte CH347IfNum;             // USB接口号: CH347T: IF0: UART;   IF1: SPI/IIC/JTAG/GPIO
                                            // CH347F: IF0: UART0;   IF1: UART1;   IF2: SPI/IIC/JTAG/GPIO
        public byte DataUpEndp;             // 批量上传端点地址
        public byte DataDnEndp;             // 批量下传端点地址
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 64)]
        public Int16[] ProductString;       // USB产品字符串
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 64)]
        public Int16[] ManufacturerString;  // USB厂商字符串
        public uint WriteTimeout;           // USB写超时
        public uint ReadTimeout;            // USB读超时
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 64)]
        public Int16[] FuncDescStr;         // 接口功能描述符
        public byte FirewareVer;            // 固件版本, 十六进制值
    }
    //public DEV_INFOR mDeviceInforS = new DEV_INFOR();
    //DEV_INFOR mPDeviceInforS = new DEV_INFOR();   // 为什么原.H文件需要这个指针？

    //[StructLayout(LayoutKind.Sequential)]         // 原.H文件这里取消了1字节对齐, C#需要取消吗？


    internal class CH347DLL
    {

        // WIN32应用层接口命令
        // public const int IOCTL_CH341_COMMAND = ( FILE_DEVICE_UNKNOWN << 16 | FILE_ANY_ACCESS << 14 | 0x0f34 << 2 | METHOD_BUFFERED )	// 专用接口
        // public const int mWIN32_COMMAND_HEAD = mOFFSET( mWIN32_COMMAND, mBuffer );	// WIN32命令接口的头长度
        public const int mCH347_MAX_NUMBER = 4;			    // 最多同时连接的CH347数
        public const int mMAX_BUFFER_LENGTH = 0x1000;       // 数据缓冲区最大长度4096
        // public const int mMAX_COMMAND_LENGTH	= ( mWIN32_COMMAND_HEAD + mMAX_BUFFER_LENGTH );	// 最大数据长度加上命令结构头的长度
        public const int mDEFAULT_BUFFER_LEN = 0x0400;      // 数据缓冲区默认长度1024
        // public const int mDEFAULT_COMMAND_LEN = ( mWIN32_COMMAND_HEAD + mDEFAULT_BUFFER_LEN );	// 默认数据长度加上命令结构头的长度

        // CH341端点地址
        public const int mCH347_ENDP_DATA_UP = 0x86;		// CH347的数据块上传端点的地址
        public const int mCH347_ENDP_DATA_DOWN = 0x06;		// CH347的数据块下传端点的地址

        // 设备层接口提供的管道操作命令
        public const int mPipeDeviceCtrl = 0x00000004;	    // CH341的综合控制管道
        public const int mPipeDataUp = 0x00000006;	        // CH341的数据块上传管道
        public const int mPipeDataDown = 0x00000007;	    // CH341的数据块下传管道

        // 应用层接口的功能代码
        public const int mFuncNoOperation = 0x00000000;	    // 无操作
        public const int mFuncGetVersion = 0x00000001;	    // 获取驱动程序版本号
        public const int mFuncGetConfig = 0x00000002;	    // 获取USB设备配置描述符
        public const int mFuncSetTimeout = 0x00000009;	    // 设置USB通讯超时
        public const int mFuncSetExclusive = 0x0000000b;	// 设置独占使用
        public const int mFuncResetDevice = 0x0000000c;	    // 复位USB设备
        public const int mFuncResetPipe = 0x0000000d;	    // 复位USB管道
        public const int mFuncAbortPipe = 0x0000000e;	    // 取消USB管道的数据请求
        public const int mFuncBufferMode = 0x00000020;      // 设定缓冲上传模式及查询缓冲区中的数据长度
        public const int mFuncBufferModeDn = 0x00000021;    // 设定缓冲下传模式及查询缓冲区中的数据长度
        public const int mFuncGetVersionEx = 0x00000022;    // 获取驱动程序版本号及芯片型号

        // USB设备标准请求代码
        public const int mUSB_CLR_FEATURE = 0x01;
        public const int mUSB_SET_FEATURE = 0x03;
        public const int mUSB_GET_STATUS = 0x00;
        public const int mUSB_SET_ADDRESS = 0x05;
        public const int mUSB_GET_DESCR = 0x06;
        public const int mUSB_SET_DESCR = 0x07;
        public const int mUSB_GET_CONFIG = 0x08;
        public const int mUSB_SET_CONFIG = 0x09;
        public const int mUSB_GET_INTERF = 0x0a;
        public const int mUSB_SET_INTERF = 0x0b;
        public const int mUSB_SYNC_FRAME = 0x0c;

        // CH341控制传输的厂商专用请求类型
        public const int mCH341_VENDOR_READ = 0xC0;		    // 通过控制传输实现的CH341厂商专用读操作
        public const int mCH341_VENDOR_WRITE = 0x40;        // 通过控制传输实现的CH341厂商专用写操作

        public const int mCH341A_CMD_I2C_STREAM = 0xAA;		// I2C接口的命令包,从次字节开始为I2C命令流
        public const int mCH341A_CMD_UIO_STREAM = 0xAB;		// UIO接口的命令包,从次字节开始为命令流
        public const int mCH341A_CMD_PIO_STREAM = 0xAE;		// PIO接口的命令包,从次字节开始为数据流

        // CH341A控制传输的厂商专用请求代码
        public const int mCH341A_BUF_CLEAR = 0xB2;          // 清除未完成的数据
        public const int mCH341A_I2C_CMD_X = 0x54;	        // 发出I2C接口的命令,立即执行
        public const int mCH341A_DELAY_MS = 0x5E;		    // 以亳秒为单位延时指定时间
        public const int mCH341A_GET_VER = 0x5F;            // 获取芯片版本

        public const int mCH341A_CMD_I2C_STM_STA = 0x74;	// I2C接口的命令流:产生起始位
        public const int mCH341A_CMD_I2C_STM_STO = 0x75;	// I2C接口的命令流:产生停止位
        public const int mCH341A_CMD_I2C_STM_OUT = 0x80;	// I2C接口的命令流:输出数据,位5-位0为长度,后续字节为数据,0长度则只发送一个字节并返回应答
        public const int mCH341A_CMD_I2C_STM_IN = 0xC0;		// I2C接口的命令流:输入数据,位5-位0为长度,0长度则只接收一个字节并发送无应答
        public const int mCH341A_CMD_I2C_STM_MAX = (0x3F < mCH347_PACKET_LENGTH) ? 0X3F : mCH347_PACKET_LENGTH; // I2C接口的命令流单个命令输入输出数据的最大长度
        public const int mCH341A_CMD_I2C_STM_SET = 0x60;	// I2C接口的命令流:设置参数,位2=SPI的I/O数(0=单入单出,1=双入双出),位1位0=I2C速度(00=低速,01=标准,10=快速,11=高速)
        public const int mCH341A_CMD_I2C_STM_US = 0x40;		// I2C接口的命令流:以微秒为单位延时,位3-位0为延时值
        public const int mCH341A_CMD_I2C_STM_MS = 0x50;		// I2C接口的命令流:以亳秒为单位延时,位3-位0为延时值
        public const int mCH341A_CMD_I2C_STM_DLY = 0x0F;	// I2C接口的命令流单个命令延时的最大值
        public const int mCH341A_CMD_I2C_STM_END = 0x00;	// I2C接口的命令流:命令包提前结束

        public const int mCH341A_CMD_UIO_STM_IN = 0x00;		// UIO接口的命令流:输入数据D7-D0
        public const int mCH341A_CMD_UIO_STM_DIR = 0x40;	// UIO接口的命令流:设定I/O方向D5-D0,位5-位0为方向数据
        public const int mCH341A_CMD_UIO_STM_OUT = 0x80;	// UIO接口的命令流:输出数据D5-D0,位5-位0为数据
        public const int mCH341A_CMD_UIO_STM_US = 0xC0;		// UIO接口的命令流:以微秒为单位延时,位5-位0为延时值
        public const int mCH341A_CMD_UIO_STM_END = 0x20;	// UIO接口的命令流:命令包提前结束

        public const int MAX_DEVICE_PATH_SIZE = 128;		// 设备名称的最大字符数
        public const int MAX_DEVICE_ID_SIZE = 64;			// 设备ID的最大字符数

        // 驱动接口
        public const int CH347_USB_VENDOR = 0;
        public const int CH347_USB_HID = 2;
        public const int CH347_USB_VCP = 3;

        // CH347_USB_VENDOR支持CH341/7
        public const int CHIP_TYPE_CH341 = 0;
        public const int CHIP_TYPE_CH347 = 1;
        public const int CHIP_TYPE_CH347F = 2;
        public const int CHIP_TYPE_CH347T = CHIP_TYPE_CH347;

        // 芯片功能接口类型
        public const int CH347_FUNC_UART = 0;
        public const int CH347_FUNC_SPI_IIC = 1;
        public const int CH347_FUNC_JTAG_IIC = 2;
        public const int CH347_FUNC_JTAG_IIC_SPI = 3;   // CH347F

        public const int DEFAULT_READ_TIMEOUT = 500;    // 默认读超时毫秒数
        public const int DEFAULT_WRITE_TIMEOUT = 500;   // 默认写超时毫秒数

        public const int mCH341_PACKET_LENGTH = 64;
        public const int mCH347_PACKET_LENGTH = 512;    // CH347支持的数据包的长度

        // CH347各模式公用函数, 支持CH347所有模式下的打开/关闭/USB读/USB写, 包含HID
        // 打开USB设备
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347OpenDevice")]
        public static extern IntPtr CH347OpenDevice(    // 打开CH347设备, 返回句柄, 出错则无效
                                    uint iIndex);       // 指定CH347设备序号, 0对应第一个设备
        // 关闭USB设备
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347CloseDevice")]
        public static extern bool CH347CloseDevice(     // 关闭CH347设备
                                  uint iIndex);         // 指定CH347设备序号
        // 获取设备信息
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347CloseDevice")]
        public static extern bool CH347GetDeviceInfor(  // 获取设备信息
                                  uint iIndex,          // 指定CH347设备序号
                                  ref DEV_INFOR DevInformation); // 设备信息结构体

        // 获取CH347芯片类型:CHIP_TYPE_CH347T/CHIP_TYPE_CH347F
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347GetChipType")]
        public static extern byte CH347GetChipType(     // 获取芯片类型
                                  uint iIndex);         // 指定设备序号
        
        // 获取驱动版本、库版本、设备版本、芯片类型(CH341(FS)/CH347HS)
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347GetVersion")]
        public static extern bool CH347GetVersion(      // 获取版本信息
                                  uint iIndex,          // 指定设备序号
                                  ref char iDriverVer,  // 驱动版本
                                  ref char iDLLVer,     // 库版本
                                  ref char ibcdDevice,  // 设备版本
                                  ref char iChipType);  // 芯片类型CHIP_TYPE_CH341/7

        // 设备事件通知回调程序
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        public delegate void mPCH347_NOTIFY_ROUTINE(    // 设备事件通知回调程序
                                  uint iEventStatus);   // 设备事件和当前状态(在下行定义): 0=设备拔出事件, 3=设备插入事件
        
        public const int CH347_DEVICE_ARRIVA = 3;	    // 设备插入事件, 已经插入
        public const int CH347_DEVICE_REMOVE_PEND = 1;  // 设备将要拔出
        public const int CH347_DEVICE_REMOVE = 0;       // 设备拔出事件, 已经拔出

        // 设定设备事件通知程序
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347SetDeviceNotify")]
        public static extern bool CH347SetDeviceNotify( // 设定设备事件通知程序
                                  uint iIndex,          // 指定CH341设备序号, 0对应第一个设备
                                  string iDeviceID,     // 可选参数, 指向字符串, 指定被监控的设备的ID, 字符串以\0终止
                                  mPCH347_NOTIFY_ROUTINE iNotifyRoutine); // 指定设备事件回调程序, 为NULL则取消事件通知, 否则在检测到事件时调用该程序

        // 读取USB数据块
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347ReadData")]
        public static extern bool CH347ReadData(        // 读取USB数据块
                                  uint iIndex,          // 指定设备序号
                                  ref byte[] oBuffer,   // 指向一个足够大的缓冲区, 用于保存读取的数据
                                  ref uint ioLength);   // 指向长度单元, 输入时为准备读取的长度, 返回后为实际读取的长度

        // 写取USB数据块
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347WriteData")]
        public static extern bool CH347WriteData(       // 写取USB数据块
                                  uint iIndex,          // 指定设备序号
                                  ref byte[] iBuffer,   // 指向一个缓冲区, 放置准备写出的数据
                                  ref uint ioLength);   // 指向长度单元, 输入时为准备写出的长度, 返回后为实际写出的长度
        
        // 设置USB数据读写的超时
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347SetTimeout")]
        public static extern bool CH347SetTimeout(      // 设置USB数据读写的超时
                                  uint iIndex,          // 指定设备序号
                                  uint iWriteTimeout,   // 指定USB写出数据块的超时时间, 以毫秒mS为单位, 0xFFFFFFFF指定不超时(默认值)
                                  uint iReadTimeout);   // 指定USB读取数据块的超时时间, 以毫秒mS为单位, 0xFFFFFFFF指定不超时(默认值)


        /*****************SPI*******************/
        // SPI控制器初始化
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347SPI_Init")]
        public static extern bool CH347SPI_Init(          // SPI控制器初始化
                                  uint iIndex,            // 指定设备序号
                                  ref SPI_CONFIG SpiCfg); // SPI配置结构体

        // CH347F设置SPI时钟频率
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347SPI_SetFrequency")]
        public static extern bool CH347SPI_SetFrequency( // CH347F设置SPI时钟频率
                                  uint iIndex,           // 指定设备序号
                                  uint iSpiSpeedHz);     // iSpiSpeedHz= 0=60MHz,   1=30MHz,    2=15MHz,    3=7.5MHz,
                                                         // (SPI速率选择) 4=3.75MHz, 5=1.875MHz, 6=937.5KHz, 7=468.75KHz

        // CH347F设置SPI支持数据位数
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347SPI_SetDataBits")]
        public static extern bool CH347SPI_SetDataBits( // CH347F设置SPI支持数据位数
                                  uint iIndex,          // 指定设备序号
                                  byte iDataBits);      // SPI支持数据位数iDataBits= 0:8bit, 1:16bit


        // 获取SPI控制器配置信息
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347SPI_GetCfg")]
        public static extern bool CH347SPI_GetCfg(        // 获取SPI控制器配置信息
                                  uint iIndex,            // 指定设备序号
                                  ref SPI_CONFIG SpiCfg); // SPI配置结构体

        // 设置片选状态   使用前需先调用CH347SPI_Init对CS进行设置
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347SPI_ChangeCS")]
        public static extern bool CH347SPI_ChangeCS(    // 设置片选状态
                                  uint iIndex,          // 指定设备序号	
                                  byte iStatus);        // 0=撤消片选, 1=设置片选

        // 设置SPI片选
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347SPI_SetChipSelect")]
        public static extern bool CH347SPI_SetChipSelect(   // 设置SPI片选
                                  uint iIndex,              // 指定设备序号
                                  ushort iEnableSelect,     // 低八位为CS1，高八位为CS2; 字节值为1=设置CS, 为0=忽略此CS设置
                                  ushort iChipSelect,       // 低八位为CS1，高八位为CS2; 片选输出, 0=撤消片选, 1=设置片选
                                  uint iIsAutoDeativeCS,    // 低16位为CS1，高16位为CS2; 操作完成后是否自动撤消片选
                                  uint iActiveDelay,        // 低16位为CS1，高16位为CS2; 设置片选后执行读写操作的延时时间, 单位us
                                  uint iDelayDeactive);     // 低16位为CS1，高16位为CS2; 撤消片选后执行读写操作的延时时间, 单位us

        // SPI4写数据
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347SPI_Write")]
        public static extern bool CH347SPI_Write(   // SPI4写数据
                                  uint iIndex,      // 指定设备序号	
                                  uint iChipSelect, // 片选控制, 位7为0则忽略片选控制, 位7为1进行片选操作
                                  uint iLength,     // 准备传输的数据字节数	
                                  uint iWriteStep,  // 准备读取的单个块的长度
                                  byte[] ioBuffer); // 指向一个缓冲区, 放置准备从MOSI写出的数据

        // SPI4读数据   无需先写数据,效率较CH347SPI_WriteRead高很多
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347SPI_Read")]
        public static extern bool CH347SPI_Read(      // SPI4读数据(高效）
                                  uint iIndex,        // 指定设备序号	
                                  uint iChipSelect,   // 片选控制, 位7为0则忽略片选控制, 位7为1进行片选操作
                                  uint oLength,       // 准备发出的字节数
                                  uint iLength,       // 准备读入的数据字节数	
                                  byte[] ioBuffer);   // 指向一个缓冲区, 放置准备从DOUT写出的数据, 返回后是从DIN读入的数据

        // SPI4读写数据
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347SPI_WriteRead")]
        public static extern bool CH347SPI_WriteRead( // SPI4读写数据
                                  uint iIndex,        // 指定设备序号
                                  uint iChipSelect,   // 片选控制, 位7为0则忽略片选控制, 位7为1则操作片选
                                  uint iLength,       // 准备传输的数据字节数
                                  byte[] ioBuffer);   // 指向一个缓冲区, 放置准备从DOUT写出的数据, 返回后是从DIN读入的数据

        // 处理4线SPI数据流
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347StreamSPI4")]
        public static extern bool CH347StreamSPI4(    // 处理4线SPI数据流
                                  uint iIndex,        // 指定设备序号
                                  uint iChipSelect,   // 片选控制, 位7为0则忽略片选控制, 位7为1则操作片选
                                  uint iLength,       // 准备传输的数据字节数
                                  byte[] ioBuffer);   // 指向一个缓冲区, 放置准备从DOUT写出的数据, 返回后是从DIN读入的数据


        /*****************JTAG******************/
        // CH347DLL.H 中JTAG相关接口(Line325 - Line451)未使用, 此CS文件暂未包括该部分内容, 如有需要可自行重写

        /*****************GPIO******************/
        // CH347DLL.H 中GPIO相关接口(Line493 - Line492)未使用, 此CS文件暂未包括该部分内容, 如有需要可自行重写

        /*****************I2C*******************/
        // CH347DLL.H 中I2C相关接口(Line548 - Line576)未使用, 此CS文件暂未包括该部分内容, 如有需要可自行重写


        /**************HID/VCP串口**************/
        // 打开CH347串口
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347Uart_Open")]
        public static extern IntPtr CH347Uart_Open(   // 打开CH347串口, 返回句柄, 出错则无效
                                    uint iIndex);     // 指定CH347设备序号, 0对应第一个设备

        // 关闭CH347串口
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347Uart_Close")]
        public static extern bool CH347Uart_Close(    // 关闭CH347串口
                                    uint iIndex);     // 指定CH347设备序号, 0对应第一个设备

        // 设定设备事件通知程序
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347Uart_SetDeviceNotify")]
        public static extern bool CH347Uart_SetDeviceNotify( // 设定设备事件通知程序
                                  uint iIndex,               // 指定设备序号, 0对应第一个设备
                                  string iDeviceID,          // 可选参数, 指向字符串, 指定被监控的设备的ID, 字符串以\0终止
                                  mPCH347_NOTIFY_ROUTINE iNotifyRoutine); // 指定设备事件回调程序, 为NULL则取消事件通知, 否则在检测到事件时调用该程序

        // 获取UART硬件配置
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347Uart_GetCfg")]
        public static extern bool CH347Uart_GetCfg(      // 获取UART硬件配置
                                  uint iIndex,           // 指定设备序号
                                  ref uint BaudRate,     // 波特率
                                  ref byte ByteSize,     // 数据位数(5,6,7,8,16)
                                  ref byte Parity,       // 校验位(0:None; 1:Odd; 2:Even; 3:Mark; 4:Space)
                                  ref byte StopBits,     // 停止位数(0:1停止位; 1:1.5停止位; 2:2停止位)
                                  ref byte ByteTimeout); // 字节超时

        // 设置UART硬件配置
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347Uart_Init")]
        public static extern bool CH347Uart_Init(       // 设置UART硬件配置
                                  uint iIndex,          // 指定设备序号
                                  uint BaudRate,        // 波特率
                                  byte ByteSize,        // 数据位数(5,6,7,8,16)
                                  byte Parity,          // 校验位(0:None; 1:Odd; 2:Even; 3:Mark; 4:Space)
                                  byte StopBits,        // 停止位数(0:1停止位; 1:1.5停止位; 2:2停止位)
                                  byte ByteTimeout);    // 字节超时时间, 单位100uS

        // 设置UART数据读写的超时
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347Uart_SetTimeout")]
        public static extern bool CH347Uart_SetTimeout( // 设置USB数据读写的超时
                                  uint iIndex,          // 指定设备序号
                                  uint iWriteTimeout,   // 指定USB写出数据块的超时时间, 以毫秒mS为单位, 0xFFFFFFFF指定不超时(默认值)
                                  uint iReadTimeout);   // 指定USB读取数据块的超时时间, 以毫秒mS为单位, 0xFFFFFFFF指定不超时(默认值)

        // UART读取数据块
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347Uart_Read")]
        public static extern bool CH347Uart_Read(       // UART读取数据块
                                  uint iIndex,          // 指定设备序号
                                  ref byte[] oBuffer,   // 指向一个足够大的缓冲区, 用于保存读取的数据
                                  ref uint ioLength);   // 指向长度单元, 输入时为准备读取的长度, 返回后为实际读取的长度

        // UART写出数据块
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347Uart_Write")]
        public static extern bool CH347Uart_Write(      // UART写出数据块
                                  uint iIndex,          // 指定设备序号
                                  ref byte[] iBuffer,   // 指向一个足够大的缓冲区, 用于保存准备写出的数据
                                  ref uint ioLength);   // 指向长度单元, 输入时为准备写出的长度, 返回后为实际写出的长度

        // 查询读缓冲区有多少字节未取
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347Uart_QueryBufUpload")]
        public static extern bool CH347Uart_QueryBufUpload( // 查询读缓冲区有多少字节未取
                                  uint iIndex,              // 指定设备序号
                                  ref Int64 RemainBytes);   // 读缓冲区未取字节数

        // 获取设备信息
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347Uart_GetDeviceInfor")]
        public static extern bool CH347Uart_GetDeviceInfor(      // 获取设备信息
                                  uint iIndex,                   // 指定设备序号
                                  ref DEV_INFOR DevInformation); // 设备信息结构体

        public enum EEPROM_TYPE
        {
            ID_24C01,
            ID_24C02,
            ID_24C04,
            ID_24C08,
            ID_24C16,
            ID_24C32,
            ID_24C64,
            ID_24C128,
            ID_24C256,
            ID_24C512,
            ID_24C1024,
            ID_24C2048,
            ID_24C4096
        };

        // 从EEPROM中读取数据块,速度约56K字节
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347ReadEEPROM")]
        public static extern bool CH347ReadEEPROM(       // 从EEPROM中读取数据块,速度约56K字节
                                  uint iIndex,           // 指定CH347设备序号
                                  EEPROM_TYPE iEepromID, // 指定EEPROM型号
                                  uint iAddr,            // 指定数据单元的地址
                                  uint iLength,          // 准备读取的数据字节数
                                  ref byte[] oBuffer);   // 指向一个缓冲区,返回后是读入的数据
        
        // 向EEPROM中写入数据块
        [DllImport("CH347DLL.DLL", EntryPoint = "CH347WriteEEPROM")]
        public static extern bool CH347WriteEEPROM(      // 向EEPROM中写入数据块
                                  uint iIndex,           // 指定CH347设备序号
                                  EEPROM_TYPE iEepromID, // 指定EEPROM型号
                                  uint iAddr,            // 指定数据单元的地址
                                  uint iLength,          // 准备写出的数据字节数
                                  ref byte[] iBuffer);   // 指向一个缓冲区,放置准备写出的数据

    }
}
